@@ -29,9 +29,9 @@ try:
 except ImportError:
     has_pygments = False
 
-# The version required by the tests is the version specified and installed in the 'pygments' tox env.
-# In any environment where the PYGMENTS_VERSION environment variable is either not defined or doesn't
-# match the version of Pygments installed, all tests which rely in pygments will be skipped.
+# The version required by the tests is the version specified and installed in the `pygments` tox environment.
+# In any environment where the `PYGMENTS_VERSION` environment variable is either not defined or doesn't
+# match the version of Pygments installed, all tests which rely in Pygments will be skipped.
 required_pygments_version = os.environ.get('PYGMENTS_VERSION', '')
 
 
@@ -54,7 +54,7 @@ class TestCodeHiliteClass(TestCase):
 
     def test_codehilite_defaults(self):
         if has_pygments:
-            # Odd result as no lang given and a single comment is not enough for guessing.
+            # Odd result as no `lang` given and a single comment is not enough for guessing.
             expected = (
                 '<div class="codehilite"><pre><span></span><code><span class="err"># A Code Comment</span>\n'
                 '</code></pre></div>'
@@ -98,7 +98,7 @@ class TestCodeHiliteClass(TestCase):
 
     def test_codehilite_set_lang(self):
         if has_pygments:
-            # Note an extra `<span class="x">` is added to end of code block when lang explicitly set.
+            # Note an extra `<span class="x">` is added to end of code block when `lang` explicitly set.
             # Compare with expected output for `test_guess_lang`. Not sure why this happens.
             expected = (
                 '<div class="codehilite"><pre><span></span><code><span class="cp">&lt;?php</span> '
@@ -122,7 +122,7 @@ class TestCodeHiliteClass(TestCase):
                 '</code></pre></div>'
             )
         else:
-            # Note that without pygments there is no way to check that the language name is bad.
+            # Note that without Pygments there is no way to check that the language name is bad.
             expected = (
                 '<pre class="codehilite"><code class="language-unkown">'
                 '&lt;?php print(&quot;Hello World&quot;); ?&gt;\n'
@@ -273,7 +273,7 @@ class TestCodeHiliteClass(TestCase):
                 '</code></pre></div>'
             )
         else:
-            # TODO: Implement linenostart for no-pygments. Will need to check what JS libs look for.
+            # TODO: Implement `linenostart` for no-Pygments. Will need to check what JavaScript libraries look for.
             expected = (
                 '<pre class="codehilite"><code class="language-text linenums">plain text\n'
                 '</code></pre>'
@@ -354,11 +354,27 @@ class TestCodeHiliteExtension(TestCase):
         if has_pygments and pygments.__version__ != required_pygments_version:
             self.skipTest(f'Pygments=={required_pygments_version} is required')
 
+        # Define a custom Pygments formatter (same example in the documentation)
+        if has_pygments:
+            class CustomAddLangHtmlFormatter(pygments.formatters.HtmlFormatter):
+                def __init__(self, lang_str='', **options):
+                    super().__init__(**options)
+                    self.lang_str = lang_str
+
+                def _wrap_code(self, source):
+                    yield 0, f'<code class="{self.lang_str}">'
+                    yield from source
+                    yield 0, '</code>'
+        else:
+            CustomAddLangHtmlFormatter = None
+
+        self.custom_pygments_formatter = CustomAddLangHtmlFormatter
+
     maxDiff = None
 
     def testBasicCodeHilite(self):
         if has_pygments:
-            # Odd result as no lang given and a single comment is not enough for guessing.
+            # Odd result as no `lang` given and a single comment is not enough for guessing.
             expected = (
                 '<div class="codehilite"><pre><span></span><code><span class="err"># A Code Comment</span>\n'
                 '</code></pre></div>'
@@ -629,7 +645,7 @@ class TestCodeHiliteExtension(TestCase):
 
     def testUnknownOption(self):
         if has_pygments:
-            # Odd result as no lang given and a single comment is not enough for guessing.
+            # Odd result as no `lang` given and a single comment is not enough for guessing.
             expected = (
                 '<div class="codehilite"><pre><span></span><code><span class="err"># A Code Comment</span>\n'
                 '</code></pre></div>'
@@ -676,3 +692,73 @@ class TestCodeHiliteExtension(TestCase):
             expected,
             extensions=[CodeHiliteExtension(pygments_style="native", noclasses=True)]
         )
+
+    def testFormatterLangStr(self):
+        if has_pygments:
+            expected = (
+                '<div class="codehilite"><pre><span></span><code class="language-python">'
+                '<span class="c1"># A Code Comment</span>\n'
+                '</code></pre></div>'
+            )
+        else:
+            expected = (
+                '<pre class="codehilite"><code class="language-python"># A Code Comment\n'
+                '</code></pre>'
+            )
+
+        self.assertMarkdownRenders(
+            '\t:::Python\n'
+            '\t# A Code Comment',
+            expected,
+            extensions=[
+                CodeHiliteExtension(
+                    guess_lang=False,
+                    pygments_formatter=self.custom_pygments_formatter
+                )
+            ]
+        )
+
+    def testFormatterLangStrGuessLang(self):
+        if has_pygments:
+            expected = (
+                '<div class="codehilite"><pre><span></span>'
+                '<code class="language-js+php"><span class="cp">&lt;?php</span> '
+                '<span class="k">print</span><span class="p">(</span>'
+                '<span class="s2">&quot;Hello World&quot;</span>'
+                '<span class="p">);</span> <span class="cp">?&gt;</span>\n'
+                '</code></pre></div>'
+            )
+        else:
+            expected = (
+                '<pre class="codehilite"><code>&lt;?php print(&quot;Hello World&quot;); ?&gt;\n'
+                '</code></pre>'
+            )
+        # Use PHP as the the starting `<?php` tag ensures an accurate guess.
+        self.assertMarkdownRenders(
+            '\t<?php print("Hello World"); ?>',
+            expected,
+            extensions=[CodeHiliteExtension(pygments_formatter=self.custom_pygments_formatter)]
+        )
+
+    def testFormatterLangStrEmptyLang(self):
+        if has_pygments:
+            expected = (
+                '<div class="codehilite"><pre><span></span>'
+                '<code class="language-text"># A Code Comment\n'
+                '</code></pre></div>'
+            )
+        else:
+            expected = (
+                '<pre class="codehilite"><code># A Code Comment\n'
+                '</code></pre>'
+            )
+        self.assertMarkdownRenders(
+            '\t# A Code Comment',
+            expected,
+            extensions=[
+                CodeHiliteExtension(
+                    guess_lang=False,
+                    pygments_formatter=self.custom_pygments_formatter,
+                )
+            ]
+        )
